@@ -23,6 +23,11 @@ require 'forwardable'
 
 module Async
 	module IO
+		# Convert a Ruby ::IO object to a wrapped instance:
+		def self.try_convert(io, &block)
+			Generic::WRAPPERS[io.class].wrap(io, &block)
+		end
+		
 		# Represents an asynchronous IO within a reactor.
 		class Generic < Wrapper
 			extend Forwardable
@@ -33,7 +38,7 @@ module Async
 				# @!macro [attach] wrap_blocking_method
 				#   @method $1
 				#   Invokes `$2` on the underlying {io}. If the operation would block, the current task is paused until the operation can succeed, at which point it's resumed and the operation is completed.
-				def wrap_blocking_method(new_name, method_name, invert: true, &block)
+				def wrap_blocking_method(new_name, method_name, invert: true)
 					define_method(new_name) do |*args|
 						async_send(method_name, *args)
 					end
@@ -94,17 +99,9 @@ module Async
 						@io.__send__(*args, exception: false)
 					end
 				end
-			else
-				def async_send(*args)
-					async do
-						@io.__send__(*args)
-					end
-				end
-			end
-			
-			def async
-				while true
-					begin
+				
+				def async
+					while true
 						result = yield
 						
 						case result
@@ -115,10 +112,42 @@ module Async
 						else
 							return result
 						end
-					rescue ::IO::WaitReadable
-						wait_readable
-					rescue ::IO::WaitWritable
-						wait_writable
+					end
+				end
+			elsif RUBY_VERSION >= "2.1"
+				def async_send(*args)
+					async do
+						@io.__send__(*args)
+					end
+				end
+				
+				def async
+					while true
+						begin
+							return yield
+						rescue ::IO::WaitReadable, ::IO::EAGAINWaitReadable
+							wait_readable
+						rescue ::IO::WaitWritable, ::IO::EAGAINWaitWritable
+							wait_writable
+						end
+					end
+				end
+			else
+				def async_send(*args)
+					async do
+						@io.__send__(*args)
+					end
+				end
+				
+				def async
+					while true
+						begin
+							return yield
+						rescue ::IO::WaitReadable
+							wait_readable
+						rescue ::IO::WaitWritable
+							wait_writable
+						end
 					end
 				end
 			end
